__author__ = 'GHajba'

import argparse
import mdxml
from wordpress_xmlrpc import Client
from wordpress_xmlrpc import WordPressPost
from wordpress_xmlrpc.methods import posts
from os.path import expanduser
from file_utils import write_line_at_beginning, read_file_lines


def convert_file(filename):
    input_lines = read_file_lines(filename)
    starter = 0
    title = None
    categories = None
    tags = None
    id = None
    for line in input_lines:
        if not line.startswith("["):
            break
        starter += 1
        if line.startswith("[title]"):
            title = line[7:]
        elif line.startswith("[categories]"):
            categories = line[12:].split(",")
        elif line.startswith("[tags]"):
            tags = line[6:].split(",")
        elif line.startswith("[id]"):
            id = line[4:]
    content = mdxml.convert_lines(input_lines[starter:])
    return id, title, categories, tags, content


def send_to_wordpress(id, title, categories, tags, content, configuration):
    if len(content.strip()) == 0:
        return

    client = Client(configuration['endpoint'], configuration["username"], configuration['password'])

    if id:
        post = client.call(posts.GetPost(id))
        pass
    else:
        post = WordPressPost()
    post.content = content
    if title is not None:
        post.title = title
    if post.title is None:
        post.title = 'My post'
    post.terms_names = {
        'post_tag': tags,
        'category': categories,
    }

    if id:
        client.call(posts.EditPost(post.id, post))
    else:
        post.id = client.call(posts.NewPost(post))

    print "Blog post with id " + post.id + " was successfully sent to WordPress."
    return post.id


def add_post_id_to_original(filename, id):
    write_line_at_beginning(filename, "[id] " + id)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("-c","--config",
                        help="The full path of the configuration file storing the XML-RPC endpoint, username and password. Per default the application looks at your home folder and searches for wpedit.conf")
    parser.add_argument("post_file", help="The full path of the input file to send to WordPress.")
    parser.add_argument("-m", "--mdconf", help="The full path of the md-to-xml conversion-extension file")
    args = parser.parse_args()

    configuration = {}
    config_file = expanduser("~")+'/wpedit.conf'
    if args.config:
        config_file = args.config
    execfile(config_file, configuration)

    mdxml.init()
    id, title, categories, tags, content = convert_file(args.post_file)
    post_id = send_to_wordpress(id, title, categories, tags, content, configuration)
    if not id and post_id:
        add_post_id_to_original(args.post_file, post_id)


# ## This is the main module of the markdown to XML converter python module
# ## (c) Gabor Laszlo Hajba 2015
"""
The main idea behind this app is to take a markdown file, parse it line-by-line and create an XML output
along the defined rules of the conversion.
"""

import re

from markdown import markdown

from configuration import Configuration
from file_utils import read_file_lines

configuration = Configuration()


def convert_line(line):
    """
    This method converts a line of markdown to specified XML. Currently hard-coded.

    The algorithm is really a wood-cutter: replace all occurrences of the markers and correct the closing tags.

    If one tag is not closed, it gets closed at the end of the line -- so currently there are no multi-line tags
    available.

    :param line: the line to convert
    :return: the line formatted as XML
    """
    global configuration
    xmlline = line

    for marker in configuration.get_normal():
        config_element = configuration.get_configuration(marker)
        marker_count = line.count(marker)

        if marker_count == 1 and configuration.alter_multiline(marker):
            xmlline = config_element.get_begin() if configuration.is_multiline() else config_element.get_end()
            xmlline += "\n"
            placeholder = ""
            if len(line.strip()[len(marker):]) > 0:
                conf = eval(line.strip()[len(marker):])
                for key in conf:
                    placeholder += " " + key + "=\"" + conf[key] + "\""
            return xmlline.format(placeholder), True

        if configuration.is_multiline():
            return line, True

        if marker_count == 0:
            continue
        xmlline = line.strip().replace(marker, config_element.get_begin())
        if xmlline.count(config_element.get_begin()) % 2 != 0:
            xmlline += config_element.get_begin()
            marker_occurrences = find_marker_occurrences(xmlline, config_element.get_begin())
            for i in marker_occurrences[1::2]:
                xmlline = xmlline[:i[0]] \
                          + (config_element.get_end() if config_element.get_end() is not None else "") + xmlline[i[1]:]

    return xmlline, False


def convert_lines(lines):
    final_text = ""
    temp = ""
    for line in lines:
        result, final = convert_line(line)
        if final:
            if len(temp) != 0:
                final_text += markdown(temp) + "\n"
                temp = ""
            final_text += result
        else:
            temp += result

    if len(temp) != 0:
        final_text += markdown(temp) + "\n"
    return final_text


def find_marker_occurrences(xmlline, marker):
    return [(a.start(), a.end()) for a in list(re.finditer(re.escape(marker), xmlline))]


def convert_md_file(filename):
    return convert_lines(read_file_lines(filename))


def write_xml_file(filename, content):
    output_file = codecs.open(filename, "w",
                              encoding="utf-8",
                              errors="xmlcharrefreplace")
    output_file.write(content)
    output_file.flush()
    output_file.close()


def init():
    global configuration
    configuration.load_configuration()


if __name__ == "__main__":
    init()
    import os

    write_xml_file(os.path.join(os.path.dirname(__file__), "../testfile.xml"),
                   convert_md_file(os.path.join(os.path.dirname(__file__), "../testfile.md")))


__author__ = 'GHajba'

from codecs import open


def write_line_at_beginning(filename, line):
    input_file = open(filename, mode="r+", encoding="utf-8")
    content = input_file.read()
    input_file.seek(0, 0)
    input_file.write(line+"\n"+content)
    input_file.close()


def read_file_as_one(filename):
    input_file = open(filename, mode="r", encoding="utf-8")
    file_content = input_file.read()
    input_file.close()
    return file_content


def read_file_lines(filename):
    input_file = open(filename, mode="r", encoding="utf-8")
    input_lines = input_file.readlines()
    input_file.close()
    return input_lines


__author__ = 'GHajba'
from distutils.core import setup
from disttest import test

setup(
    name='wpedit',
    packages=['wpedit'],
    version='0.2',
    description='Sends (offline) articles written in MarkDown to WordPress via the XML-RPC API',
    author='Gabor Laszlo Hajba',
    author_email='gabor.hajba@gmail.com',
    url='https://github.com/ghajba/wp-editor',
    download_url='https://github.com/ghajba/wp-editor/tarball/0.1',
    keywords=['wordpress', 'offline', 'edit', 'markdown', 'xml'],
    classifiers=['Programming Language :: Python',
                 'License :: OSI Approved :: MIT License', ],
    options={
        'test':{
            'test_dir':['tests']
        },
    },
    cmdclass = {'test':test},
    requires=[('Markdown'), ("python.wordpress.xmlrpc")],
)


"""
This file loads and maintains the configuration of the application
"""

import os

class ConfigurationElement:
    def __init__(self, marker, begin, end=None, multiline=False, starter=False, complex=False):
        self.__marker = marker
        self.__begin = begin
        self.__end = end
        self.__multiline = multiline
        self.__starter = starter
        self.__complex = complex

    def __str__(self):
        return self.__marker + " --> " + self.__begin + " " + (self.__end if self.__end is not None else "") \
               + (" multi-line" if self.__multiline else "") \
               + (" starter" if self.is_starter() else "") \
               + (" complex" if self.is_complex() else "")

    def get_begin(self):
        return self.__begin

    def get_marker(self):
        return self.__marker

    def get_end(self):
        return self.__end

    def is_multiline(self):
        return self.__multiline

    def is_complex(self):
        return self.__complex

    def is_starter(self):
        return self.__starter


# TODO add possibility to have multiple categories of configurations like starter, complex and multiline
class Configuration:
    def __init__(self):
        self.__multiline_added = False
        self.__multiline_marker = None
        self.__configuration = {}
        self.__markers = []
        self.__default = None

    def load_configuration(self, config_file=os.path.join(os.path.dirname(__file__), "mdxml.conf")):
        """
        :param config_file: the file where the configuration is stored
        """
        config = open(config_file)
        config_lines = config.readlines()
        for line in config_lines:
            if len(line.strip()) == 0:
                continue
            params = line.split()
            marker = params[0]
            begin = params[1]
            multiline = False
            starter = False
            complex = False
            if len(params) >= 3:
                end = params[2]
            else:
                end = None
                # TODO add variable configuration for fourth parameter, for example M for multiline, S for starter, C for complex
            if len(params) > 3:
                if "M" == params[3]:
                    multiline = True
                elif "S" == params[3]:
                    starter = True
                elif "C" == params[3]:
                    complex = True
            if "default" == marker:
                self.__default = ConfigurationElement(marker, begin, end)
            else:
                self.__configuration[marker] = ConfigurationElement(marker, begin, end, multiline, starter, complex)

    def get_configuration(self, marker=None):
        """
        :param marker: the marker to look up in the configuration
        :return: the loaded Configuration element or None if the marker is not present
        """
        if marker is None:
            return sorted(self.__configuration, key=len, reverse=True)
        if marker in self.__configuration:
            return self.__configuration[marker]
        return None

    def get_default(self):
        return self.__default

    def print_configuration(self):
        for key in self.__configuration:
            print self.__configuration[key]

    def is_multiline(self):
        return self.__multiline_added

    def alter_multiline(self, actual_marker):
        if not self.__configuration[actual_marker].is_multiline():
            return False
        if self.__multiline_added and not self.__multiline_marker == actual_marker:
            return False
        if self.__multiline_added:
            self.__multiline_marker = None
        else:
            self.__multiline_marker = actual_marker
        self.__multiline_added = not self.__multiline_added
        return True

    def get_starters(self):
        """This method returns all configuration elements which can appear only at the beginning of a line"""
        return sorted([k for k, v in self.__configuration.items() if v.is_starter()], key=len, reverse=True)

    def get_complex(self):
        """This method returns all the complex configuration elements"""
        return sorted([k for k, v in self.__configuration.items() if v.is_complex()], key=len, reverse=True)

    def get_normal(self):
        """This method returns the normal configuration elements which are not complex and not starter"""
        return sorted([k for k, v in self.__configuration.items() if not v.is_starter() and not v.is_complex()],key=len, reverse=True)